/**
 * RapidMiner GmbH
 *
 * Copyright (C) 2020 by RapidMiner GmbH and the contributors
 *
 * Complete list of developers available at our web site:
 *
 *      www.rapidminer.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package com.rapidminer.extension.connection_conversion.connection.gui;

import static com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurator.OPTIONS;
import static com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurator.OPTION_A;
import static com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurator.OPTION_B;
import static com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurator.OPTION_C;
import static com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurator.PARAMETER_FIRST_B;
import static com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurator.PARAMETER_OPTION;
import static com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurator.PARAMETER_PASSWORD;
import static com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurator.PARAMETER_SECOND_B;
import static com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurator.PARAMETER_SINGLE_A;
import static com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurator.PARAMETER_USE_FOR_C;

import java.awt.Window;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JComponent;

import com.rapidminer.connection.ConnectionInformation;
import com.rapidminer.connection.gui.DefaultConnectionGUI;
import com.rapidminer.connection.gui.components.ConnectionParameterCheckBox;
import com.rapidminer.connection.gui.components.InjectableComponentWrapper;
import com.rapidminer.connection.gui.model.ConnectionModel;
import com.rapidminer.connection.gui.model.ConnectionParameterGroupModel;
import com.rapidminer.connection.gui.model.ConnectionParameterModel;
import com.rapidminer.repository.RepositoryLocation;

/**
 * Implementation of UI for {@link com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurable SharedDataConfigurable}.
 *
 * @since 9.6.0
 */
public class SharedDataGUI extends DefaultConnectionGUI {

	public SharedDataGUI(Window parent, ConnectionInformation connection, RepositoryLocation location, boolean editable) {
		super(parent, connection, location, editable);
	}

	@Override
	public JComponent getComponentForGroup(ConnectionParameterGroupModel groupModel, ConnectionModel connectionModel) {
		ConnectionParameterModel optionParameter = getOrCreateParameter(groupModel, PARAMETER_OPTION);
		ConnectionParameterModel singleAParameter = getOrCreateParameter(groupModel, PARAMETER_SINGLE_A);
		ConnectionParameterModel firstBParameter = getOrCreateParameter(groupModel, PARAMETER_FIRST_B);
		ConnectionParameterModel secondBParameter = getOrCreateParameter(groupModel, PARAMETER_SECOND_B);
		ConnectionParameterModel useForCParameter = getOrCreateParameter(groupModel, PARAMETER_USE_FOR_C);
		singleAParameter.enabledProperty().bind(optionParameter.valueProperty().isEqualTo(OPTION_A));
		firstBParameter.enabledProperty().bind(optionParameter.valueProperty().isEqualTo(OPTION_B));
		secondBParameter.enabledProperty().bind(optionParameter.valueProperty().isEqualTo(OPTION_B));
		useForCParameter.enabledProperty().bind(optionParameter.valueProperty().isEqualTo(OPTION_C));

		return super.getComponentForGroup(groupModel, connectionModel);
	}

	/**
	 * Get or create the parameter with the given name
	 */
	private ConnectionParameterModel getOrCreateParameter(ConnectionParameterGroupModel group, String paramName) {
		ConnectionParameterModel parameter = group.getParameter(paramName);
		if (parameter != null) {
			return parameter;
		}
		boolean isEncrypted = PARAMETER_PASSWORD.equals(paramName);
		group.addOrSetParameter(paramName, "", isEncrypted, null, true);
		return group.getParameter(paramName);
	}

	@Override
	protected JComponent getParameterLabelComponent(String type, ConnectionParameterModel parameter) {
		return visibilityWrapper(super.getParameterLabelComponent(type, parameter), parameter);
	}

	@Override
	protected JComponent getParameterInputComponent(String type, ConnectionParameterModel parameter) {
		switch (parameter.getName()) {
			case PARAMETER_OPTION:
				return parameter.isEditable() ?
						InjectableComponentWrapper.getInjectableCombobox(parameter, OPTIONS, OPTION_B) :
						super.getParameterInputComponent(type, parameter);
			case PARAMETER_USE_FOR_C:
				return new ConnectionParameterCheckBox(parameter);
			default:
				return super.getParameterInputComponent(type, parameter);
		}
	}

	/** Wrap the component with both an information icon as well as a visbility wrapper */
	@Override
	protected JComponent wrapInformationIcon(ConnectionModel connectionModel, ConnectionParameterModel parameter, JComponent component) {
		return visibilityWrapper(super.wrapInformationIcon(connectionModel, parameter, component), parameter);
	}

	@Override
	public List<ConnectionParameterModel> getInjectableParameters(ConnectionParameterGroupModel group) {
		List<ConnectionParameterModel> injectables = new ArrayList<>(group.getParameters());
		injectables.removeIf(p -> !p.isEnabled());
		injectables.removeIf(p -> p.getName().equals(PARAMETER_USE_FOR_C));
		return injectables;
	}
}
