/**
 * RapidMiner GmbH
 *
 * Copyright (C) 2020 by RapidMiner GmbH and the contributors
 *
 * Complete list of developers available at our web site:
 *
 *      www.rapidminer.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package com.rapidminer.extension.connection_conversion;

import static com.rapidminer.connection.CreateI18NKeysForConnectionHandler.appendOrReplaceConnectionKeys;
import static com.rapidminer.connection.CreateI18NKeysForConnectionHandler.connectionAdaptionHandlerDefaultValues;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.Properties;
import java.util.function.BiFunction;

/**
 * Helper class to update Gui*.properties file with connection keys
 */
public class I18NHelper {

	/**
	 * Run this to update the GUI*.properties file
	 *
	 * @see com.rapidminer.connection.CreateI18NKeysForConnectionHandler CreateI18NKeysForConnectionHandler
	 */
	public static void main(String[] args) throws IOException {
		Runnable initializer = PluginInitConnectionConversion::initPlugin;
		String namespace = PluginInitConnectionConversion.CONVERSION_NAMESPACE;
		Path propertyPath = Paths.get("src/main/resources/com/rapidminer/extension/resources/i18n/GUIConnectionConversion.properties").toAbsolutePath();
		BiFunction<String, Properties, String> defaultValues = connectionAdaptionHandlerDefaultValues(namespace);
		Path newPath = appendOrReplaceConnectionKeys(initializer, namespace, propertyPath, defaultValues);
		if (newPath == null) {
			return;
		}
		Files.copy(Files.newInputStream(newPath), propertyPath, StandardCopyOption.REPLACE_EXISTING);
	}
}
